/*
Copyright 2012 Embedded Data Systems, LLC

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/
package eds_xml_reader;
/*
 * XMLReaderGUI.java
 *
 * Created on Oct 24, 2012, 2:05:10 PM
 */
import java.io.*;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.*;

import javax.xml.parsers.*;
import org.xml.sax.InputSource;
import org.w3c.dom.*;

import javax.swing.*;
import java.util.Timer;
import java.util.TimerTask;
import java.util.Date;

/**
 * Provides the GUI interface and most of the code to read and manage XML from a server
 *
 * @version @Revision: 1.00 $
 * @since Build {1.00} (10 2012)
 * @author Larry Wood
 * @author $Author: nbweb $, (this version)
 *
 */
public class XMLReaderGUI extends javax.swing.JFrame {
	private static final long serialVersionUID = 1L;

	private ArrayList <String>EUIs;
	private String EUIName;
	private String FileName;

	/** Creates new form XMLReaderGUI */
	public XMLReaderGUI() {
		initComponents();
	}

	/**
	 * This method is called from within the constructor to
	 * initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is
	 * always regenerated by the Form Editor.
	 */
	@SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        choice1 = new java.awt.Choice();
        message = new java.awt.TextArea();
        panel1 = new java.awt.Panel();
        lblHeading = new java.awt.Label();
        lblStep1 = new java.awt.Label();
        lblStep2 = new java.awt.Label();
        MultiDevice = new java.awt.Checkbox();
        lblURL = new java.awt.Label();
        url = new java.awt.TextField();
        lblTag1 = new java.awt.Label();
        Tag1 = new java.awt.TextField();
        lblTag2 = new java.awt.Label();
        lblTag3 = new java.awt.Label();
        lblEUI = new java.awt.Label();
        lblLogFile = new javax.swing.JLabel();
        Tag2 = new java.awt.TextField();
        Tag3 = new java.awt.TextField();
        eui = new java.awt.TextField();
        File = new java.awt.TextField();
        Start = new java.awt.Button();
        Log = new java.awt.Button();
        Quit = new java.awt.Button();
        Units = new java.awt.Choice();
        label1 = new java.awt.Label();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("EDS XML Reader - v1.00");

        message.setEditable(false);

        lblHeading.setText("Read and parse XML data from an OW Server or MeshNet Controller");

        lblStep1.setText("Step 1: Select the reading type using the checkbox");

        lblStep2.setText("Step 2: Fill in the remaining fields");

        MultiDevice.setLabel("Read 1 tag from multiple devices (disables Tag 2, Tag 3 and EUI)");
        MultiDevice.setState(true);
        MultiDevice.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                MultiDeviceItemStateChanged(evt);
            }
        });

        lblURL.setText("URL");

        url.setText("http://192.168.1.14/details.xml");

        lblTag1.setText("Tag 1");

        Tag1.setText("Temperature");

        lblTag2.setText("Tag 2");

        lblTag3.setText("Tag 3");

        lblEUI.setText("EUI");

        lblLogFile.setText("Log File");

        Tag2.setEnabled(false);

        Tag3.setEnabled(false);

        eui.setEnabled(false);

        File.setEditable(false);
        File.setEnabled(false);

        Start.setLabel("Begin");
        Start.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                StartActionPerformed(evt);
            }
        });

        Log.setLabel("Select Log File");
        Log.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                LogActionPerformed(evt);
            }
        });

        Quit.setLabel("Quit");
        Quit.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                QuitActionPerformed(evt);
            }
        });

        Units.add("Metric");
        Units.add("English");

        label1.setText("Units");

        org.jdesktop.layout.GroupLayout panel1Layout = new org.jdesktop.layout.GroupLayout(panel1);
        panel1.setLayout(panel1Layout);
        panel1Layout.setHorizontalGroup(
            panel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(panel1Layout.createSequentialGroup()
                .addContainerGap()
                .add(panel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(lblTag1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(lblTag2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(panel1Layout.createSequentialGroup()
                        .add(panel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(panel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                                .add(org.jdesktop.layout.GroupLayout.LEADING, lblHeading, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                .add(org.jdesktop.layout.GroupLayout.LEADING, lblStep1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                .add(org.jdesktop.layout.GroupLayout.LEADING, lblStep2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                .add(org.jdesktop.layout.GroupLayout.LEADING, MultiDevice, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                .add(org.jdesktop.layout.GroupLayout.LEADING, panel1Layout.createSequentialGroup()
                                    .add(panel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                        .add(lblTag3, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                        .add(lblLogFile)
                                        .add(lblURL, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                                    .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                    .add(panel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                        .add(Tag3, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 555, Short.MAX_VALUE)
                                        .add(Tag1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 555, Short.MAX_VALUE)
                                        .add(url, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 555, Short.MAX_VALUE)
                                        .add(Tag2, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 555, Short.MAX_VALUE)
                                        .add(eui, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 555, Short.MAX_VALUE)
                                        .add(org.jdesktop.layout.GroupLayout.TRAILING, File, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 555, Short.MAX_VALUE)
                                        .add(panel1Layout.createSequentialGroup()
                                            .add(Log, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                            .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 79, Short.MAX_VALUE)
                                            .add(label1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                            .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                            .add(Units, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 79, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                            .add(110, 110, 110)
                                            .add(Start, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 60, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                            .add(18, 18, 18)
                                            .add(Quit, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 56, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))))
                            .add(panel1Layout.createSequentialGroup()
                                .add(lblEUI, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                .add(590, 590, 590)))
                        .add(119, 119, 119)))
                .addContainerGap())
        );
        panel1Layout.setVerticalGroup(
            panel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, panel1Layout.createSequentialGroup()
                .add(panel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, Tag2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, panel1Layout.createSequentialGroup()
                        .add(lblHeading, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(lblStep1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(lblStep2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(MultiDevice, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(panel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(lblURL, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .add(url, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(panel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(org.jdesktop.layout.GroupLayout.TRAILING, lblTag1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                            .add(org.jdesktop.layout.GroupLayout.TRAILING, Tag1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(lblTag2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(panel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                    .add(lblTag3, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(Tag3, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(panel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                    .add(lblEUI, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(eui, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .add(14, 14, 14)
                .add(panel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                    .add(File, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(lblLogFile))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(panel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                    .add(panel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                        .add(Quit, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .add(Log, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .add(Start, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .add(label1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 20, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(Units, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 24, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .add(26, 26, 26))
        );

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, message, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 651, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, panel1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 651, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(panel1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .add(message, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 454, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

	private void StartActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_StartActionPerformed
		try {
			int FoundDevice = 0;
			// Get the XML file
			GetXMLFromDevice getXML = new GetXMLFromDevice();
			String results = getXML.GetDataUsingHTTP(url.getText());
			// If we actually got it, parse it and setup for reading it every second
			if(results != null) {
				DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
				DocumentBuilder db = dbf.newDocumentBuilder();
				InputSource is = new InputSource();
				is.setCharacterStream(new StringReader(results));
				Document doc = db.parse(is);
				XMLParsingHelpers XMLHelpers = new XMLParsingHelpers();

				// We need to determine the unique ID name for the device, the OW Server uses
				// ROMId, the MeshNet controller uses EUI.
				EUIName = new String();
				NodeList spnodes = doc.getElementsByTagName("EUI");
				Element name = (Element) spnodes.item(0);
				if(name != null) {
					EUIName = "EUI";
				}
				else {
					EUIName = "ROMId";
				}

				// If multiple device checkbox is checked, we will try to read multiple devices for the same tag
				if(MultiDevice.getState() == true) {
					spnodes = doc.getElementsByTagName(Tag1.getText());
					int numnodes = spnodes.getLength();
					printMessage("Reading " + numnodes + " devices for the tag '" + Tag1.getText() + "'", false);
					String outputStr = new String();
					outputStr = "Date/Time,Uptime";

					// Reading the XML file each time and displaying the value in the order of the file is a
					// bad idea because the order of the devices in the file can change if power is lost or a device is
					// removed and reinstalled. We have to store the EUI's and specifically retrieve the value for
					// each one. Here we create the list of EUIs.
					EUIs = new ArrayList<String>();
					spnodes = doc.getElementsByTagName(EUIName);
					for(int i=0; i < numnodes; i++) {
						name = (Element) spnodes.item(i);
						EUIs.add(XMLHelpers.getCharacterDataFromElement(name));
						outputStr = outputStr + "," + EUIs.get(i);
					}
					// Lets write a line to the output that identifies each column
					printMessage(outputStr, false);
					FoundDevice++;
				}
				// We are reading one device for one or more tags
				else {
					Document docEUI = XMLHelpers.getNodeListByEUI(eui.getText(), doc);
					if(docEUI == null) {
						printMessage("Device (" + eui.getText() + ") could not be found in the XML file", false);
						return;
					}

					// If Tag1 cannot be found in the XML file, display an error
					// If Tag2 or Tag3 can't be found, don't worry about it
					NodeList nodes = docEUI.getElementsByTagName(Tag1.getText());
					name = (Element) nodes.item(0);
					String tag1 = XMLHelpers.getCharacterDataFromElement(name);
					if(tag1 == null) {
						printMessage("The Tag1 field (" + Tag1.getText() + ") does not exist in the XMLfile", false);
						return;
					}

					// Write a title and a line to identify each column
					String outputStr = Tag1.getText();
					if(!"".equals(Tag2.getText())) {
						outputStr += "," + Tag2.getText();
					}
					if(!"".equals(Tag3.getText())) {
						outputStr += "," + Tag3.getText();
					}
					printMessage("Reading " + eui.getText() + " for " + outputStr, false);
					outputStr = "Date/Time,Uptime," + outputStr;
					printMessage(outputStr, false);
					FoundDevice++;
				}

				// If all went well, start up the timer for 1 second ticks
				if(FoundDevice > 0) {
					// Disable the fields so they cannot be changed once we started
					Start.setEnabled(false);
					Log.setEnabled(false);
					MultiDevice.setEnabled(false);
					url.setEnabled(false);
					Tag1.setEnabled(false);
					Tag2.setEnabled(false);
					Tag3.setEnabled(false);
					eui.setEnabled(false);
					Timer timer = new Timer();
					timer.schedule(new TimerTick(), 1000, 1000);
				}
			}
			else {
				printMessage("Could not read XML File", false);
			}
		} catch (Exception e) {
			printMessage("Error reading XML File", false);
//			e.printStackTrace();
		}
	}//GEN-LAST:event_StartActionPerformed

	// Come here every second, read the XML file and write the necessary data to the output
	private class TimerTick extends TimerTask {
		private Document doc;

		@Override
		public void run() {
			try {
				// Get the XML file
				GetXMLFromDevice getXML = new GetXMLFromDevice();
				String results = getXML.GetDataUsingHTTP(url.getText());
				if(results != null) {
					DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
					DocumentBuilder db = dbf.newDocumentBuilder();

					InputSource is = new InputSource();
					is.setCharacterStream(new StringReader(results));

					doc = db.parse(is);
					XMLParsingHelpers XMLHelpers = new XMLParsingHelpers();

					// Print uptime
					NodeList nodes = doc.getElementsByTagName("UpTime");
					Element uptime = (Element) nodes.item(0);
					if(uptime == null) {
						nodes = doc.getElementsByTagName("PollCount");
						uptime = (Element) nodes.item(0);
					}
					String outputStr = XMLHelpers.getCharacterDataFromElement(uptime);

					// Reading one tag from multiple devices. We stored the EUI's of the devices to
					// read at startup to make certain we always read them in order. Now all we have
					// to do is isolate each device, then read the tag of interest.
					if(MultiDevice.getState() == true) {
						// Loop through the list of EUI's
						for(int i=0; i < EUIs.size(); i++) {
							// Get the XML data for just this one device, doing so by EUI
							Document docEUI = XMLHelpers.getNodeListByEUI(EUIs.get(i), doc);
							// Get the field from the XML for the device
							nodes = docEUI.getElementsByTagName(Tag1.getText());
							Element name = (Element) nodes.item(0);
							String v = XMLHelpers.getCharacterDataFromElement(name);
							// Convert
							v = getConvertedValue(v, name.getAttribute("Units"), Units.getSelectedItem());
							// Add it to the output string
							outputStr += "," + v;
						}
					}
					// Device monitoring
					else {
						Document docEUI = XMLHelpers.getNodeListByEUI(eui.getText(), doc);
						// We tested the first XML tag, so get it
						nodes = docEUI.getElementsByTagName(Tag1.getText());
						Element name1 = (Element) nodes.item(0);
						String v1 = XMLHelpers.getCharacterDataFromElement(name1);
						v1 = getConvertedValue(v1, name1.getAttribute("Units"), Units.getSelectedItem());
						// Try to get the second XML tag
						nodes = docEUI.getElementsByTagName(Tag2.getText());
						Element name2 = (Element) nodes.item(0);
						String v2 = XMLHelpers.getCharacterDataFromElement(name2);
						v2 = getConvertedValue(v2, name2.getAttribute("Units"), Units.getSelectedItem());
						// Try to get the third XML tag
						nodes = docEUI.getElementsByTagName(Tag3.getText());
						Element name3 = (Element) nodes.item(0);
						String v3 = XMLHelpers.getCharacterDataFromElement(name3);
						v3 = getConvertedValue(v3, name3.getAttribute("Units"), Units.getSelectedItem());
						// Create the output string, don't put Tag2 or Tag3 if we couldn't find them
						outputStr += "," + v1;
						if(name2 != null) {
							outputStr += "," + v2;
						}
						if(name3 != null) {
							outputStr += "," + v3;
						}
					}

				printMessage(outputStr, true);
				}
			} catch (Exception e) {
				printMessage("Timer Exception", true);
//				e.printStackTrace();
			}
		}
	}

	public void printMessage(String msg, boolean TimeStamp) {
		String output = "";
		if(TimeStamp == true) {
			Date now = new Date();
			SimpleDateFormat formatter = new SimpleDateFormat("yy.MM.dd_h:mm.ss,");
			output = formatter.format(now);
		}
		message.append(output + msg + '\n');
		if(FileName != null) {
			try {
				FileWriter fstream = new FileWriter(FileName, true);
				BufferedWriter out = new BufferedWriter(fstream);
				out.write(output + msg + '\n');
				out.close();
			}
			catch (Exception e) {
				printMessage("File Writing Exception", true);
//				e.printStackTrace();
			}
		}
	}

	// Values come in metric, this routine converts them to english is set to do so
	private String getConvertedValue(String val, String attribute, String units) {
		double cVal;

		// Convert value to int, exit if it is not a number
		try {
			cVal = Float.parseFloat(val);
		}
		catch(NumberFormatException nfe) {
			return val;
		}

		// If temp
		if("Centigrade".equals(attribute)) {
			if("English".equals(units)) {
				cVal = (cVal * (9.0/5.0)) + 32.0;
			}
			DecimalFormat df = new DecimalFormat("#.0");
			return df.format(cVal);
		}

		// If barometric pressure
		if("Millibars".equals(attribute)) {
			if("English".equals(units)) {
				cVal *= 0.0295301;
			}
			DecimalFormat df = new DecimalFormat("#.000");
			return df.format(cVal);
		}

		// Do nothing
		return val;
	}

	private void QuitActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_QuitActionPerformed
		setVisible(false);
		dispose();
		System.exit(0);
	}//GEN-LAST:event_QuitActionPerformed

	private void MultiDeviceItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_MultiDeviceItemStateChanged
		if(MultiDevice.getState() == true) {
			Tag2.setEnabled(false);
			Tag3.setEnabled(false);
			eui.setEnabled(false);
			lblTag2.setEnabled(false);
			lblTag3.setEnabled(false);
			lblEUI.setEnabled(false);
		}
		else {
			Tag2.setEnabled(true);
			Tag3.setEnabled(true);
			eui.setEnabled(true);
			lblTag2.setEnabled(true);
			lblTag3.setEnabled(true);
			lblEUI.setEnabled(true);
		}
	}//GEN-LAST:event_MultiDeviceItemStateChanged

	private void LogActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_LogActionPerformed
		JFileChooser fc = new JFileChooser();
		int rv = fc.showOpenDialog(this);

		if(rv == JFileChooser.APPROVE_OPTION) {
			File file = fc.getSelectedFile();
			File.setText(file.getPath());
			FileName = file.getPath();
			file.delete();
		}
	}//GEN-LAST:event_LogActionPerformed

	public void RunGUI() {
		/* Set the Nimbus look and feel */
		//<editor-fold defaultstate="collapsed" desc=" Look and feel setting code (optional) ">
        /* If Nimbus (introduced in Java SE 6) is not available, stay with the default look and feel.
		 * For details see http://download.oracle.com/javase/tutorial/uiswing/lookandfeel/plaf.html
		 */
		try {
			for (javax.swing.UIManager.LookAndFeelInfo info : javax.swing.UIManager.getInstalledLookAndFeels()) {
				if ("Nimbus".equals(info.getName())) {
					javax.swing.UIManager.setLookAndFeel(info.getClassName());
					break;
				}
			}
		} catch (ClassNotFoundException ex) {
			java.util.logging.Logger.getLogger(XMLReaderGUI.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		} catch (InstantiationException ex) {
			java.util.logging.Logger.getLogger(XMLReaderGUI.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		} catch (IllegalAccessException ex) {
			java.util.logging.Logger.getLogger(XMLReaderGUI.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		} catch (javax.swing.UnsupportedLookAndFeelException ex) {
			java.util.logging.Logger.getLogger(XMLReaderGUI.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		}
		//</editor-fold>

		/* Create and display the form */
		java.awt.EventQueue.invokeLater(new Runnable() {

			@Override
			public void run() {
				new XMLReaderGUI().setVisible(true);
			}
		});
	}
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private java.awt.TextField File;
    private java.awt.Button Log;
    private java.awt.Checkbox MultiDevice;
    private java.awt.Button Quit;
    private java.awt.Button Start;
    private java.awt.TextField Tag1;
    private java.awt.TextField Tag2;
    private java.awt.TextField Tag3;
    private java.awt.Choice Units;
    private java.awt.Choice choice1;
    private java.awt.TextField eui;
    private java.awt.Label label1;
    private java.awt.Label lblEUI;
    private java.awt.Label lblHeading;
    private javax.swing.JLabel lblLogFile;
    private java.awt.Label lblStep1;
    private java.awt.Label lblStep2;
    private java.awt.Label lblTag1;
    private java.awt.Label lblTag2;
    private java.awt.Label lblTag3;
    private java.awt.Label lblURL;
    private java.awt.TextArea message;
    private java.awt.Panel panel1;
    private java.awt.TextField url;
    // End of variables declaration//GEN-END:variables
}
